from typing import override
from astropy.units import Unit
from gwpy.timeseries import TimeSeries
from h5py import File, Group, Dataset

from backscattering_experimental_data.section import Section
from backscattering_experimental_data.metadata import (
    MetadataComponent as Metadata,
)
from backscattering_experimental_data.utils import dataset_to_timeseries


class Component(Section):
    """
    A Component is an element of the interferometer that is associated
    with a movement sensor. Right now, we only store movement in the z
    axis.
    Attributes
    ----------
    movement : gwpy.timeseries.TimeSeries[length]
        Corrected movement of the component
    """

    def __init__(self, metadata: Metadata, sensor: TimeSeries):
        """
        Parameters
        ----------
        metadata : vda.package.metadata.Component
            Metadata associated to this data
        sensor : gwpy.timeseries.TimeSeries[Volt]
            Sensor data of the component (in Volt, not in meter !)
        Warnings
        --------
        It's an internal class, of whose instances are not produced via
        a direct call, but by using helper functions
        See Also
        --------
        bed.MetadataComponent : Associated metadata type
        """
        self.metadata = metadata
        self.sensor = sensor
        self.movement = self.sensor * self.metadata.calibration_factor

    @override
    @staticmethod
    def from_hdf5(store: File | Group) -> "Component":
        sensor_dataset = store.get("sensor")
        if not isinstance(sensor_dataset, Dataset):
            raise ValueError(
                "sensor should exists, and not be {}".format(
                    type(sensor_dataset)
                )
            )
        sensor = dataset_to_timeseries(sensor_dataset)
        return Component(
            Metadata.from_hdf5(store),
            sensor=sensor,
        )

    @override
    def to_hdf5(self, store: File | Group) -> Group | File:
        self.metadata.to_hdf5(store)
        dataset_store = store.create_dataset(
            name="sensor", data=self.sensor
        )
        if type(self.sensor.unit) is Unit:
            dataset_store.attrs["unit"] = self.sensor.unit.to_string()
        return store
